{lib, ourpkgs, ...}:
let
  gridlib = import ../../lib;

  rawConfig = lib.importJSON ./config.json;
  config = rawConfig // {
    sshUsers = import ./public-keys/users.nix;

    # Convert relative paths to absolute so library code can resolve names
    # correctly.
    publicKeyPath = toString ./. + "/${rawConfig.publicKeyPath}";
    privateKeyPath = toString ./. + "/${rawConfig.privateKeyPath}";
  };

  # Configure deployment management authorization for all systems in the grid.
  grid-config = {
    _module.args = { inherit ourpkgs; };
    imports = [ gridlib.base ];
    services.private-storage.deployment = {
      authorizedKey = builtins.readFile "${config.publicKeyPath}/deploy_key.pub";
      gridName = "local";
    };
    services.private-storage.sshUsers = config.sshUsers;
    grid = {
      inherit (config) publicKeyPath privateKeyPath; 
    };
  };

  node = { monitor-ip, public-ip }: { pkgs, lib, ...}: {
    image.name = "localhost/nixos-test";
    nixos.useSystemd = true;

    #service.useHostStore = true;
    image.includeStorePaths = false;
    image.includeNixDb = false;
    service.volumes = ["nix:/nix:ro"];

    out.service = {
      networks.privatestorage.ipv4_address = public-ip;
    };
    nixos.configuration = {
      imports = [
        # Include the module from morph, which defines the
        # `deployment` option.
        "${pkgs.morph.lib}/options.nix"
        gridlib.hardware-docker
        ./secrets.nix
        grid-config
      ];
      config = {
        system.stateVersion = lib.mkDefault "19.09";
        monitoringvpnIPv4 = monitor-ip;
	services.openssh.enable = lib.mkForce false;
      };
    };
  };
    
  storage-node = { monitor-ip, public-ip }@args: { pkgs, lib, ...}: {
    imports = [ (node args) ];
    nixos.configuration = {
      imports = [
        gridlib.storage
          (gridlib.customize-storage config)
      ];
    };
  };
in
{
  services.storage1 = storage-node { monitor-ip = "172.23.23.12"; public-ip = "10.88.2.30"; };
  services.storage2 = storage-node { monitor-ip = "172.23.23.13"; public-ip = "10.88.2.31"; };
  services.payment = { pkgs, lib, ...}: {
    imports = [ (node { monitor-ip = "172.23.23.11"; public-ip = "10.88.2.32"; }) ];
    config = {
      nixos.configuration = {config, pkgs, ...}: {
        imports = [
          gridlib.issuer
            (gridlib.customize-issuer (rawConfig // {
              monitoringvpnIPv4 = "172.23.23.11";
              }))
        ];
        environment.etc = {
          "secrets/payments-localdev-ssl" = {
            mode = "direct-symlink";
            source = "${config.grid.privateKeyPath}/payments-localdev-ssl";
          };
          "stripe/config.toml" = let
            toml = pkgs.formats.toml {};
            api-key = lib.fileContents config.deployment.secrets.stripe-secret-key.source;
          in {
            source = toml.generate "stripe.conf" {
              default = {
                device_name = "localdev";
                test_mode_api_key = api-key;
              };
            };
          };
        };
        services.private-storage-issuer.tls = lib.mkForce false;
        environment.systemPackages = [ pkgs.stripe-cli ];
      };
      service.ports = ["8080:80"];
    };
  };
  docker-compose.raw = {
    networks.privatestorage = { name = "privatestorage"; external = true; };
    volumes.nix = { name = "nix"; external = true; };
  };
}
